package cc.fascinated.bat.features.autorole.profile;

import cc.fascinated.bat.common.Serializable;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.service.DiscordService;
import com.google.gson.Gson;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import net.dv8tion.jda.api.entities.Role;
import org.bson.Document;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Setter
@Getter
@NoArgsConstructor
public class AutoRoleProfile extends Serializable {
    private static final int DEFAULT_MAX_ROLES = 10;
    private static final int PREMIUM_MAX_ROLES = 25;

    /**
     * The roles to assign when a user joins
     */
    private List<String> roleIds;

    /**
     * Gets the maximum amount of roles that can be set in the guild
     *
     * @param guild the guild to check
     * @return the amount of role slots
     */
    public static int getMaxRoleSlots(BatGuild guild) {
        if (guild.getPremiumProfile().hasPremium()) {
            return PREMIUM_MAX_ROLES;
        }
        return DEFAULT_MAX_ROLES;
    }

    /**
     * Gets the amount of role slots in use
     *
     * @return the amount
     */
    public int getRoleSlotsInUse() {
        return getRoles().size();
    }

    /**
     * Adds a role to assign when a user joins
     *
     * @param roleId the role ID to add
     */
    public void addRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        this.roleIds.add(roleId);
    }

    /**
     * Removes a role from being assigned when a user joins
     *
     * @param roleId the role ID to remove
     */
    public void removeRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        this.roleIds.remove(roleId);
    }

    /**
     * Checks if a role is set to be assigned when a user joins
     *
     * @param roleId the role ID to check
     * @return whether the role is set
     */
    public boolean hasRole(String roleId) {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        return this.roleIds.contains(roleId);
    }

    /**
     * Gets the roles to assign when a user joins
     *
     * @return the roles
     */
    public List<Role> getRoles() {
        if (this.roleIds == null) {
            this.roleIds = new ArrayList<>();
        }
        List<Role> roles = new ArrayList<>();
        for (String id : this.roleIds) {
            Role role = DiscordService.JDA.getRoleById(id);
            if (role != null) {
                roles.add(role);
            }
        }
        return roles;
    }

    @Override
    public void reset() {
        roleIds.clear();
    }

    @Override
    public void load(Document document, Gson gson) {
        roleIds = document.getList("roleIds", String.class);
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        document.put("roleIds", roleIds);
        return document;
    }
}
