package cc.fascinated.bat.features.drag.command;

import cc.fascinated.bat.command.BatCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.TimerUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.drag.DragRequest;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.GuildVoiceState;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.concrete.VoiceChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import net.dv8tion.jda.api.interactions.commands.build.OptionData;
import net.dv8tion.jda.api.interactions.components.ActionRow;
import net.dv8tion.jda.api.interactions.components.buttons.Button;
import org.springframework.stereotype.Component;

import java.time.Duration;
import java.util.HashSet;
import java.util.Set;

/**
 * Handles requests to be moved to a voice channel.
 * Author: Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "request", description = "Request to be moved to a voice channel")
public class RequestSubCommand extends BatCommand implements EventListener {
    /**
     * A list of join requests
     */
    public static final Set<DragRequest> JOIN_REQUESTS = new HashSet<>();

    private final long requestTimeout = Duration.ofMinutes(30).toMillis();
    private final long checkInterval = Duration.ofSeconds(10).toMillis();

    public RequestSubCommand() {
        super.addOptions(new OptionData(OptionType.USER, "user", "The user you want to join", true));

        TimerUtils.scheduleRepeating(() -> {
            Set<DragRequest> toRemove = new HashSet<>();
            for (DragRequest joinRequest : JOIN_REQUESTS) {
                if (System.currentTimeMillis() - joinRequest.getRequestDate().getTime() < requestTimeout) {
                    return;
                }
                // The request has timed out
                joinRequest.getInteractionHook().editOriginalEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("The request to join %s's voice channel has timed out.".formatted(joinRequest.getTarget().getAsMention()))
                        .build()).queue();
                joinRequest.getVoiceChannel().sendMessageEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("%s's request to join your voice channel has timed out.".formatted(joinRequest.getMember().getAsMention()))
                        .build()).queue();
                joinRequest.getRequestMessage().delete().queue();
                toRemove.add(joinRequest);
            }
            JOIN_REQUESTS.removeAll(toRemove);
        }, checkInterval, checkInterval);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        GuildVoiceState voiceState = member.getVoiceState();
        // Check if the user is in a voice channel
        if (voiceState == null || voiceState.getChannel() == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You are not in a voice channel.")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        OptionMapping userOption = event.getOption("user");
        assert userOption != null;

        // Check if the user is in a voice channel
        Member target = userOption.getAsMember();
        if (target == null || target.getId().equals(member.getId())) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You cannot request to join your own voice channel.")
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        // Check if the target user is in a voice channel
        GuildVoiceState targetVoiceState = target.getVoiceState();
        if (targetVoiceState == null || targetVoiceState.getChannel() == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("The user %s is not in a voice channel.".formatted(target.getAsMention()))
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        VoiceChannel targetChannel = targetVoiceState.getChannel().asVoiceChannel();
        // User is already in the target channel
        if (voiceState.getChannel().getId().equals(targetChannel.getId())) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You are already in the voice channel %s.".formatted(voiceState.getChannel().getAsMention()))
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        // Check if the user has already requested to join the target channel
        DragRequest existingRequest = JOIN_REQUESTS.stream()
                .filter(request -> request.getMember().getId().equals(member.getId()) && request.getVoiceChannel().getId().equals(targetChannel.getId()))
                .findFirst()
                .orElse(null);
        if (existingRequest != null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You have already requested to join %s's voice channel.".formatted(target.getAsMention()))
                            .build())
                    .setEphemeral(true)
                    .queue();
            return;
        }

        // Add the request to the list
        JOIN_REQUESTS.add(new DragRequest(member, target, targetChannel, event.getHook()));

        // Send the request to the target user
        targetChannel.sendMessage(target.getAsMention()).queue();
        targetChannel.sendMessageEmbeds(EmbedUtils.successEmbed()
                        .setDescription("User %s has requested to join your voice channel.".formatted(member.getAsMention()))
                        .build())
                .addComponents(ActionRow.of(
                        Button.primary("drag-request-accept", "Accept"),
                        Button.danger("drag-request-decline", "Decline")
                ))
                .queue(message -> JOIN_REQUESTS.stream()
                        .filter(r -> r.getVoiceChannel().getId().equals(targetChannel.getId()))
                        .findFirst().ifPresent(request -> request.setRequestMessage(message)));
        event.replyEmbeds(EmbedUtils.successEmbed()
                        .setDescription("Request to join %s's voice channel has been sent.".formatted(target.getAsMention()))
                        .build())
                .setComponents(ActionRow.of(Button.secondary("drag-request-cancel", "Cancel")))
                .queue();
    }
}